/***********************************************************************

          restored with the Arduino Leonardo (Arduino Micro)

           Using ArduinoJoystickLibrary by Matthew Heironimus
          https://github.com/MHeironimus/ArduinoJoystickLibrary
 **********************************************************************/
#define DEVICE_ID  5

#define STEP 12
#define DIR 11

int trimmer_position = 0;
int stepper_driver(int new_pos);

#define ANALOG_FIFO_LENGTH (12)
#define CHANNELS_NUMBER  (6)

void moving_average_filter_init(void);
short moving_average_filter(int ch, short in);

#include "Joystick.h"
/*--------------------------------------------------------
  "THE BEER-WARE LICENSE" (Revision 42):
  Alex Kostyuk wrote this code. As long as you retain this
  notice you can do whatever you want with this stuff.
  If we meet some day, and you think this stuff is worth it,
  you can buy me a beer in return.
  ----------------------------------------------------------*/
#if 0
Joystick_::Joystick_(
  uint8_t hidReportId,
  uint8_t joystickType,
  uint8_t buttonCount,
  uint8_t hatSwitchCount,
  bool includeXAxis,
  bool includeYAxis,
  bool includeZAxis,
  bool includeRxAxis,
  bool includeRyAxis,
  bool includeRzAxis,
  bool includeRudder,
  bool includeThrottle,
  bool includeAccelerator,
  bool includeBrake,
  bool includeSteering)
#endif

#define STX   '{'
#define ETX   '}'

enum {
  FLAPS_0 = 0,
  FLAPS_1 = 70,
  FLAPS_2 = 130,
  FLAPS_3 = 190,
  FLAPS_4 = 260,
  FLAPS_5 = 320,
  FLAPS_6 = 380,
  FLAPS_7 = 440
};
#define FLAPS_DELTA 25
char lever_state[8];


/*-------------------------------------------------------------
  W A R N I N G !
  Put into the file
  Arduino/hardware/arduino/avr/cores/arduino/HardwareSerial.h
  after lines:

  #ifndef HardwareSerial_h
  #define HardwareSerial_h

  these lines to increase the serial buffer size
     #define SERIAL_TX_BUFFER_SIZE 512
     #define SERIAL_RX_BUFFER_SIZE 1024
  ---------------------------------------------------------------*/
#if SERIAL_TX_BUFFER_SIZE != 512
//#error SERIAL_TX_BUFFER_SIZE still is not 512 bytes
#endif

#if SERIAL_RX_BUFFER_SIZE != 1024
//#error SERIAL_RX_BUFFER_SIZE still is not 1024 bytes
#endif


int is_reverse_activated(void);

const char hex_table[] = "0123456789ABCDEF";

static unsigned char nibble(int hex)
{
  if (hex >= '0' && hex <= '9') return (hex - '0');
  if (hex >= 'A' && hex <= 'F') return (hex - 'A' + 10);
  if (hex >= 'a' && hex <= 'f') return (hex - 'a' + 10);
  return 0;
}

static int hex2int(char *hh)
{
  int nib1 = nibble(hh[0] & 0xFF);
  int nib2 = nibble(hh[1] & 0xFF);
  return  (nib1 << 4) + nib2;
}

unsigned long hex2ulong(const char *in)
{
  unsigned long ul  = 0;
  char *wp = (char*)in;
  int i;

  for (i = 0; i < 4; i++)
  {
    ul <<= 8;
    ul += hex2int(wp);
    wp += 2;
  }

  return ul;
}








Joystick_ Joystick(JOYSTICK_DEFAULT_REPORT_ID,
                   JOYSTICK_TYPE_MULTI_AXIS, 32, 0,
                   true, /* XAxis       */
                   true, /* YAxis       */
                   true, /* ZAxis       */
                   true, /* RxAxis      */
                   true, /* RyAxis      */
                   true, /* RzAxis      */
                   false,  /* Rudder      */
                   false,  /* Throttle    */
                   false, /* Accelerator */
                   false, /* Brake       */
                   false  /* Steering    */
                  );


void setup()
{
  Serial.begin(115200);
  moving_average_filter_init();
  //discrete_task_init();
  //Joystick.setXAxisRange(0, 1023);
  //Joystick.setYAxisRange(0, 1023);
  pinMode(8, INPUT_PULLUP);
  pinMode(9, INPUT_PULLUP);
  pinMode(STEP, OUTPUT);
  pinMode(DIR, OUTPUT);
  digitalWrite(DIR, HIGH);
  digitalWrite(STEP, LOW);
  Joystick.begin(false);
}


void serial_read(void)
{
  static char buf[8];
  static int ndx = 0;
  int itmp;
  if (Serial.available() > 0)
  {
    char ch = (char)Serial.read();
    switch (ch)
    {
      case STX:
        ndx = 0;
        break;

      case ETX:

        if (buf[0] == '7')
        {
          buf[7] = 0;
          trimmer_position = atoi(&buf[3]);

        }

        if (buf[0] == 'D')
        {
          Serial.write('!');
          Serial.write(STX);
          Serial.write('F');
          itmp = DEVICE_ID;
          Serial.write(hex_table[itmp >> 4]);
          Serial.write(hex_table[itmp & 0x0F]);
          Serial.write(ETX);

        }
        else
          Serial.write('!');
        break;
      default:
        if (ndx < 8)
          buf[ndx++] = ch;

    }
  }

}

#define BUTTON_PRESSING_TIME  20;

void loop(void)
{
  static unsigned long ms_reftime = 0;
  static short cur[6] = {0, 0, 0, 0, 0, 0};
  unsigned long cur_time = millis();
  static char button0 = 0, button1 = 0, cnt = 0;
  static int timer = 100;
  serial_read();
  stepper_driver(850 - trimmer_position);



  if ((cur_time - ms_reftime) >= 1)
  {
    ms_reftime = cur_time;

    static char prev_en_A = 0;
    char en_A = digitalRead(8);
    char en_B = digitalRead(9);

    if (button0)
    {
      if (--button0 == 0)
        Joystick.releaseButton(0);

    }

    if (button1)
    {
      if (--button1 == 0)
        Joystick.releaseButton(1);

    }


    if (prev_en_A != en_A)
    {
      if (en_A)
      {
        if (en_B)
        {
          //Serial.print("0");
          if (!button0)
            Joystick.pressButton(0);
          button0 = BUTTON_PRESSING_TIME;
        }

        else
        {
          //Serial.print("1");
          if (!button1)
            Joystick.pressButton(1);
          button1 = BUTTON_PRESSING_TIME;
        }
      }
      prev_en_A = en_A;
    }

    if (++cnt == 10)
    {
      cnt = 0;
#if 1
      cur[0] = moving_average_filter(0, analogRead(A0));
      cur[1] = moving_average_filter(1, analogRead(A1));
      cur[2] = moving_average_filter(2, analogRead(A2));
      cur[3] = moving_average_filter(3, analogRead(A3));
      cur[4] = moving_average_filter(4, analogRead(A4));
      cur[5] = moving_average_filter(5, analogRead(A5));
#else
      //cur[0] = analogRead(A0);
      cur[0] = moving_average_filter(0,  analogRead(A0));
      cur[1] = analogRead(A1);
      cur[2] = analogRead(A2);
      cur[3] = analogRead(A3);
      cur[4] = analogRead(A4);
      cur[5] = analogRead(A5);
#endif
#if 0
      Serial.print(cur[0]);
      Serial.print(" ");
      Serial.print(cur[1]);
      Serial.print(" ");
      Serial.print(cur[2]);
      Serial.print(" ");
      Serial.print(cur[3]);
      Serial.print(" ");
      Serial.print(cur[4]);
      Serial.print(" ");
      Serial.println(cur[5]);
#endif


    }

    Joystick.setXAxis(cur[0]);
    Joystick.setYAxis(cur[1]);
    Joystick.setZAxis(cur[2]);
    Joystick.setRxAxis(cur[3]);
    Joystick.setRyAxis(cur[4]);
    Joystick.setRzAxis(cur[5]);

    Joystick.sendState();
  }
}




enum {
  STEPPER_POWER_UP,
  STEPPER_IDLE,
  STEPPER_GO_HOME,
  STEPPER_FOLLOW_REF
};

void one_step(int dir)
{
  digitalWrite(DIR, dir);
  digitalWrite(STEP, HIGH);
  digitalWrite(STEP, HIGH);
  digitalWrite(STEP, HIGH);
  digitalWrite(STEP, LOW);
}

int stepper_driver(int new_pos)
{
  //static int ref_time_step = 0;
  static unsigned long ref_time = 0;
  static int state = STEPPER_POWER_UP;
  static int pos, ref;
  int dir;
  if ( (millis() - ref_time) > 1)
  {
    switch (state)
    {
      case STEPPER_POWER_UP:
        pos = 0;
        ref = 2000;
        state = STEPPER_GO_HOME;
        break;
      case STEPPER_IDLE:
        ref = new_pos;
        state = STEPPER_FOLLOW_REF;
        break;

      case STEPPER_GO_HOME:
        if (pos < ref)
        {
          pos++;
          one_step(0);
        }
        else
        {
          pos = 0;
          state = STEPPER_IDLE;
        }

        break;

      case STEPPER_FOLLOW_REF:
        ref = new_pos;
        if (ref > 1700)
          ref = 1700;
        if (ref < 0)
          ref = 0;
        if (pos < ref)
        {
          pos++;
          one_step(1);
        }
        else if (pos > ref)
        {
          pos--;
          one_step(0);
        }
        else
        {
          state = STEPPER_IDLE;
        }


        break;

    }
    ref_time = millis();
  }

  return state;
}

typedef struct
{
  int   push_indx;
  int   pop_indx;
  short buff[ANALOG_FIFO_LENGTH];
} SIMPLE_FIFO;

SIMPLE_FIFO analog_fifo[CHANNELS_NUMBER];

short fifo_push(SIMPLE_FIFO *fifo, short data);
short fifo_pop(SIMPLE_FIFO *fifo, short *data);

void moving_average_filter_init(void)
{
  memset(analog_fifo, 0, sizeof(analog_fifo));
}



short moving_average_filter(int ch, short in)
{
  static short sum[CHANNELS_NUMBER] = {0, 0, 0, 0, 0, 0};
  short tmp = in;
  short prev;
  sum[ch] += in;
  if (!fifo_push(&analog_fifo[ch], in))
  {
    fifo_pop(&analog_fifo[ch], &prev);
    fifo_push(&analog_fifo[ch], in);
    sum[ch] -= prev;
    tmp = (sum[ch] / (ANALOG_FIFO_LENGTH - 1));
    return tmp;
  }


  return 0;
}

short fifo_push(SIMPLE_FIFO *fifo, short data)
{
  int indx;
  indx = fifo->push_indx + 1;
  indx = indx % ANALOG_FIFO_LENGTH;

  if (indx == fifo->pop_indx)
    return 0; // fifo is full
  fifo->buff[indx] = data;
  fifo->push_indx = indx;
  return 1;
}

short fifo_pop(SIMPLE_FIFO *fifo, short *data)
{
  int indx;
  if (fifo->push_indx == fifo->pop_indx)
    return 0; // fifo is empty
  indx = fifo->pop_indx + 1;
  indx = indx % ANALOG_FIFO_LENGTH;
  *data = fifo->buff[indx];
  fifo->pop_indx = indx;
  return 1;
}





